package com.socklabs.whoisthat.configuration;

import com.socklabs.whoisthat.facebook.CacheFriendsAfterConnectInterceptor;
import com.socklabs.whoisthat.facebook.PopupDialogConnectInterceptor;
import com.socklabs.whoisthat.mvc.controller.SinglePageConnectController;
import com.socklabs.whoisthat.twitter.CacheTwitterFriendsAfterConnectInterceptor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.*;
import org.springframework.core.env.Environment;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.encrypt.Encryptors;
import org.springframework.social.connect.Connection;
import org.springframework.social.connect.ConnectionFactoryLocator;
import org.springframework.social.connect.ConnectionRepository;
import org.springframework.social.connect.UsersConnectionRepository;
import org.springframework.social.connect.jdbc.JdbcUsersConnectionRepository;
import org.springframework.social.connect.support.ConnectionFactoryRegistry;
import org.springframework.social.connect.web.ConnectController;
import org.springframework.social.facebook.api.Facebook;
import org.springframework.social.facebook.api.impl.FacebookTemplate;
import org.springframework.social.facebook.connect.FacebookConnectionFactory;
import org.springframework.social.twitter.api.Twitter;
import org.springframework.social.twitter.api.impl.TwitterTemplate;
import org.springframework.social.twitter.connect.TwitterConnectionFactory;

import javax.inject.Inject;
import javax.sql.DataSource;

/**
 * The application configuration class.
 *
 * Make note of the use of the XXXAppConfig classes that contain properties that are specific to specific
 * profiles/environments. Specifically the ProductionAppConfig class is used to load a special social-production
 * property file only when the "production" profile is set (through the @Profile annotation). If this app grows to
 * support multiple environments, I can just pop them onto the @Import annotation below and not have to worry about
 * updating my web app initializer.
 *
 * @author ngerakines
 * @since 2/19/12
 */
@Configuration
@ComponentScan( basePackages = "com.socklabs.whoisthat", excludeFilters = @ComponentScan.Filter(Configuration.class))
@PropertySource({"classpath:/social.properties"})
@Import(ProductionAppConfig.class)
public class AppConfig {

	private final static Logger logger = LoggerFactory.getLogger(AppConfig.class);

	@Inject
	private Environment environment;

	@Inject
	private DataSource dataSource;

	@Bean
	@Scope(value="singleton", proxyMode= ScopedProxyMode.INTERFACES)
	public ConnectionFactoryLocator connectionFactoryLocator() {
		ConnectionFactoryRegistry registry = new ConnectionFactoryRegistry();
		registry.addConnectionFactory(new FacebookConnectionFactory(environment.getProperty("facebook.appid"), environment.getProperty("facebook.secret")));
		registry.addConnectionFactory(new TwitterConnectionFactory(environment.getProperty("twitter.appid"), environment.getProperty("twitter.secret")));
		return registry;
	}

	@Bean
	@Scope(value="singleton", proxyMode=ScopedProxyMode.INTERFACES)
	public UsersConnectionRepository usersConnectionRepository() {
		return new JdbcUsersConnectionRepository(dataSource, connectionFactoryLocator(), Encryptors.noOpText());
	}

	@Bean
	@Scope(value="request", proxyMode=ScopedProxyMode.INTERFACES)
	public ConnectionRepository connectionRepository() {
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		if (authentication == null) {
			throw new IllegalStateException("Unable to get a ConnectionRepository: no user signed in");
		}
		return usersConnectionRepository().createConnectionRepository(authentication.getName());
	}

	@Bean
	@Scope(value="request", proxyMode=ScopedProxyMode.INTERFACES)
	public Facebook facebook() {
		Connection<Facebook> facebook = connectionRepository().findPrimaryConnection(Facebook.class);
		logger.info("facebook {}", facebook);
		return facebook != null ? facebook.getApi() : new FacebookTemplate();
	}

	@Bean
	@Scope(value="request", proxyMode=ScopedProxyMode.INTERFACES)
	public Twitter twitter() {
		Connection<Twitter> twitter = connectionRepository().findPrimaryConnection(Twitter.class);
		return twitter != null ? twitter.getApi() : new TwitterTemplate();
	}

	@Bean
	public CacheFriendsAfterConnectInterceptor cacheFriendsAfterConnectInterceptor() {
		return new CacheFriendsAfterConnectInterceptor();
	}

	@Bean
	public CacheTwitterFriendsAfterConnectInterceptor cacheTwitterFriendsAfterConnectInterceptor() {
		return new CacheTwitterFriendsAfterConnectInterceptor();
	}

	@Bean
	public ConnectController connectController() {
		SinglePageConnectController connectController = new SinglePageConnectController(connectionFactoryLocator(), connectionRepository());
		connectController.addInterceptor(new PopupDialogConnectInterceptor());
		connectController.addInterceptor(cacheFriendsAfterConnectInterceptor());
		connectController.addInterceptor(cacheTwitterFriendsAfterConnectInterceptor());
		return connectController;
	}

}
