package com.socklabs.whoisthat.mvc;

import com.socklabs.whoisthat.mvc.model.Entity;
import com.socklabs.whoisthat.mvc.model.EntityMeta;
import com.socklabs.whoisthat.mvc.model.Tripple;
import com.socklabs.whoisthat.mvc.model.Tuple;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * Created by IntelliJ IDEA.
 * User: ngerakines
 * Date: 2/19/12
 * Time: 8:55 PM
 * To change this template use File | Settings | File Templates.
 */
public class EntityManagerImpl implements EntityManager {

	@Autowired
	private JdbcTemplate jdbcTemplate;

	@Override
	public List<Entity> getByAccount(final String account) {
		List<Entity> entities = jdbcTemplate.query("SELECT * FROM entity WHERE account = ?", new Object[]{ account }, entityRowMapper);
		backfillMeta(entities);
		backfillTags(entities);
		backfillGroups(entities);
		return entities;
	}

	public List<Tripple<String, String, String>> getAllNamesByAccount(final String account) {
		return jdbcTemplate.query("SELECT * FROM entity WHERE account = ?", new Object[]{ account }, entityIdNameRowMapper);
	}

	@Override
	public List<Tuple<String, String>> getGroupsForEntities(String... entityIds) {
		List<Tuple<String, String>> groupRecords = new ArrayList<Tuple<String, String>>();
		for (String entityId : entityIds) {
			groupRecords.addAll(jdbcTemplate.query("SELECT * FROM entity_group WHERE entity_id = ?", new Object[]{ entityId }, entityGroupRowMapper));
		}
		return groupRecords;
	}

	public List<EntityMeta> getEntityMetaByEntity(final String entityId) {
		return jdbcTemplate.query("SELECT * FROM entity_meta WHERE entity_id = ?", new Object[]{ entityId }, entityMetaRowMapper);
	}

	@Override
	public String create(final String email, final String source, final String label) {
		final String SQL = "insert into entity values (?, ?, ?, ?, CURRENT_TIMESTAMP, CURRENT_TIMESTAMP, null, null)";
		final String id = UUID.randomUUID().toString();
		jdbcTemplate.update(SQL, id, email, source, label);
		return id;
	}

	@Override
	public String addMeta(String entityId, String label, String value) {
		int count = jdbcTemplate.queryForInt("SELECT count(*) FROM entity_meta where entity_id = ? AND label = ?", new Object[]{entityId, label});
		if (count == 0) {
			final String SQL = "insert into entity_meta values (?, ?, ?, ?, CURRENT_TIMESTAMP)";
			final String id = UUID.randomUUID().toString();
			jdbcTemplate.update(SQL, id, entityId, label, value);
			return id;
		} else {
			final String SQL = "update entity_meta set note = ? where entity_id = ? AND label = ?";
			jdbcTemplate.update(SQL, value, entityId, label);
		}
		return null;
	}

	@Override
	public void setGroup(String groupId, String entityId) {
		int count = jdbcTemplate.queryForInt("SELECT count(*) FROM entity_group where entity_id = ? AND id = ?", new Object[]{entityId, groupId});
		if (count == 0) {
			final String SQL = "insert into entity_group values (?, ?, CURRENT_TIMESTAMP)";
			jdbcTemplate.update(SQL, groupId, entityId);
		} else {
			final String SQL = "update entity_group set id = ? where entity_id = ?";
			jdbcTemplate.update(SQL, groupId, entityId);
		}
	}

	@Override
	public void addTag(String entityId, String tag) {
		final String SQL = "insert into entity_tag values (?, ?, CURRENT_TIMESTAMP)";
		jdbcTemplate.update(SQL, entityId, tag);
	}

	@Override
	public List<Entity> getByAccountPaginated(String account, int offset, int count) {
		List<Entity> entities = jdbcTemplate.query("SELECT * FROM entity as entitya WHERE account = ? LIMIT ? OFFSET ?", new Object[]{ account, count, offset }, entityRowMapper);
		backfillMeta(entities);
		backfillTags(entities);
		backfillGroups(entities);
		return entities;
	}

	@Override
	public List<Entity> getByAccountAndSourcePaginated(String account, String source, int offset, int count) {
		List<Entity> entities = jdbcTemplate.query("SELECT * FROM entity as entitya WHERE account = ? AND source = ? LIMIT ? OFFSET ?", new Object[]{ account, source, count, offset }, entityRowMapper);
		backfillMeta(entities);
		backfillTags(entities);
		backfillGroups(entities);
		return entities;
	}

	private List<String> getEntityTagsByEntity(String entityId) {
		return jdbcTemplate.query("SELECT tag FROM entity_tag WHERE entity_id = ?", new Object[]{ entityId }, entityTagRowMapper);
	}

	private void backfillTags(final List<Entity> entities) {
		for (Entity entity : entities) {
			List<String> tagsByEntity = getEntityTagsByEntity(entity.getId());
			for (String tag : tagsByEntity) {
				entity.addTags(tag);
			}
		}
	}

	private void backfillMeta(final List<Entity> entities) {
		for (Entity entity : entities) {
			List<EntityMeta> entityMetaList = getEntityMetaByEntity(entity.getId());
			for (EntityMeta entityMeta : entityMetaList) {
				entity.addMetas(entityMeta);
			}
		}
	}

	private void backfillGroups(final List<Entity> entities) {
		for (Entity entity : entities) {
			List<String> entityGroupList = getEntityGroupByEntity(entity.getId());
			for (String groupId : entityGroupList) {
				entity.setGroup(groupId);
			}
		}
	}

	private List<String> getEntityGroupByEntity(String entityId) {
		return jdbcTemplate.query("SELECT id FROM entity_group WHERE entity_id = ?", new Object[]{ entityId }, entityGroupIdRowMapper);
	}

	private static final RowMapper<Entity> entityRowMapper = new RowMapper<Entity>() {
		@Override
		public Entity mapRow(ResultSet resultSet, int i) throws SQLException {
			/* created_at TIMESTAMP, updated_at TIMESTAMP, orphaned_at TIMESTAMP, deleted_at TIMESTAMP */
			Timestamp createdAt = resultSet.getTimestamp("created_at");
			Timestamp updatedAt = resultSet.getTimestamp("updated_at");
			Timestamp orphanedAt = resultSet.getTimestamp("orphaned_at");
			Timestamp deletedAt = resultSet.getTimestamp("deleted_at");
			return new Entity( /* id varchar(255), account varchar(255), source varchar(255), label varchar(255) */
					resultSet.getString("id"),
					resultSet.getString("account"),
					resultSet.getString("source"),
					resultSet.getString("label"),
					createdAt != null ? new DateTime(createdAt.getTime()) : null,
					updatedAt != null ? new DateTime(updatedAt.getTime()) : null,
					orphanedAt != null ? new DateTime(orphanedAt.getTime()) : null,
					deletedAt != null ? new DateTime(deletedAt.getTime()) : null
			);
		}
	};

	private static final RowMapper<EntityMeta> entityMetaRowMapper = new RowMapper<EntityMeta>() {
		@Override
		public EntityMeta mapRow(ResultSet resultSet, int i) throws SQLException {
			Timestamp createdAt = resultSet.getTimestamp("created_at");
			return new EntityMeta( /* (id varchar(255), entity_id varchar(255), label varchar(255), note clob); */
					resultSet.getString("id"),
					resultSet.getString("entity_id"),
					resultSet.getString("label"),
					resultSet.getString("note"),
					createdAt != null ? new DateTime(createdAt.getTime()) : null
			);
		}
	};

	private static final RowMapper<String> entityTagRowMapper = new RowMapper<String>() {
		@Override
		public String mapRow(ResultSet resultSet, int i) throws SQLException {
			return resultSet.getString("tag");
		}
	};

	private static final RowMapper<String> entityGroupIdRowMapper = new RowMapper<String>() {
		@Override
		public String mapRow(ResultSet resultSet, int i) throws SQLException {
			return resultSet.getString("id");
		}
	};

	private static final RowMapper<Tuple<String, String>> entityGroupRowMapper = new RowMapper<Tuple<String, String>>() {
		@Override
		public Tuple<String, String> mapRow(ResultSet resultSet, int i) throws SQLException {
			return new Tuple<String, String>(resultSet.getString("id"), resultSet.getString("entity_id"));
		}
	};

	private static final RowMapper<Tripple<String, String, String>> entityIdNameRowMapper = new RowMapper<Tripple<String, String, String>>() {
		@Override
		public Tripple<String, String, String> mapRow(ResultSet resultSet, int i) throws SQLException {
			return new Tripple<String, String, String>(resultSet.getString("id"), resultSet.getString("label"), resultSet.getString("source"));
		}
	};

}
